{{

Fun Game
---Base Station Component

This is a simple game that uses the Parallax Propeller
on a Stingray robot to implement a game. The challenge
of the game is to drive a robot around and "collect"
RFID tags.

This is the base station component. This portion of the
program uses two joysticks (via pots) to provide a tank
drive style of control to the robot. For feedback, a TV
screen is used to give program output. An optional (but
fun) accessory is a wireless camera and another TV
screen. Otherwise, you can just watch the robot.

This game requires two programs: one for the base station
(this program) and one for the robot (StingrayRFIDGame).

The two XBEEs need to be set up in AP mode, and able to
communicate. Interface speed on both sides is set to
57600 baud.


Created for the lab final project in CS120B, W2011.

SRLM
}}

CON

  _xinfreq = 5_000_000
  _clkmode = xtal1 + pll16x

  'channels
  lvert = 5
  lhorz = 6
  lsel = 7

  rvert = 0
  rhorz = 1
  rsel = 2

  time_step = 100 'clkfreq / time_step; aka hz for reading inputs

  tag_count = 4 'the number of tags in the game

OBJ
  com  : "FullDuplexSerialPlus"
  rfid : "FullDuplexSerialPlus"
  ADC : "MCP3208_fast"
  text : "tv_text"
  'debug: "FullDuplexSerialPlus"

VAR

  byte tags[8*11]
  byte known_tags

  byte tag[11]

  long input_function_stack[20]

  long lspeed, rspeed, button

  long str_title

  long last_time
  long game_time

PUB Main | temp, i

  str_title := string("Fun Game (c)2042")

  text.start(12)
  com.start(7, 6, 0, 57600)
  rfid.start(4, 16, 0, 2400) 'P16 is unused (demo board LED)
  ADC.start(1, 2, 0, 0)
  'debug.start(31, 30, 0, 115200)

  known_tags := 0

  waitcnt(clkfreq/4 + cnt)
  cognew(input, @input_function_stack)
  waitcnt(clkfreq/4 + cnt)

  'debug.str(string("base station..."))

  repeat i from 12 to 1
    text.str(string($A, 1, $B))
    text.out(i+1)
    text.str(string( $9, $9, $9, $A, 1, $B))
    text.out(i)
    text.str(str_title)
    waitcnt(clkfreq/10 + cnt)


  text.out(0)
  text.str(string($A, 1, $B, 1))
  text.str(str_title)
  text.str(string($A, 1, $B, 3))
  text.str(string("Game: virtually collect as many cards"))
  text.str(string($A, 8, $B, 4))
  text.str(string("possible within the time limit"))
  text.str(string($A, 1, $B, 6))
  text.str(string("Drive the robot with tank drive:"))
  text.str(string($A, 5, $B, 7))
  text.str(string("Left joystick  -> left motor speed"))
  text.str(string($A, 5, $B, 8))
  text.str(string("Right joystick -> right motor speed"))
  text.str(string($A, 1, $B, 9))
  text.str(string("Drive over a card and press the right"))
  text.str(string($A, 1, $B, 10))
  text.str(string("joystick to collect a card!"))
  text.str(string($A, 1, $B, 12))
  text.str(string("Press left joystick to begin playing!"))


  'Wait to begin game
  repeat until ((button & %10)>>1) == 1

  text.out(0) 'cls
  game_time := 0
  last_time := cnt

  'constant text
  text.str(string($A, 1, $B, 1))
  text.str(str_title)
  text.str(string($A, 19, $B, 1, "time: "))
  text.str(string($A, 1, $B, 5, "Tags found:"))

  repeat until known_tags == tag_count

    output_timer(25, 1)

    'if(button)
      'debug.str(string("button", 13))

    temp := com.rxcheck
    if temp <> -1
      repeat
        'input is RFID
        if temp == $0A
          repeat i from 0 to 9
            tag[i] := com.rx
          tag[i] := 0 ' i ==10
          quit
        if temp == -1
          quit
        temp := com.rxcheck

    if(temp == $0A)
      text.str(string($A, 1, $B, 3, $9, $9, $9)) 'clear line
      text.str(string($A, 1, $B, 3))
      text.str(string("Found RFID: "))
      text.str(@tag)
      tag_known_add(@tag)


    output_tags(1, 6)

    waitcnt(clkfreq / 10 + cnt)

  text.out(0)
  text.str(string($A, 1, $B, 1))
  text.str(str_title)
  text.str(string($A, 15, $B, 5, "Your Score:"))
  output_timer(20, 7)

  text.str(string($A, 10, $B, 12, "Press reset to play again"))

  'Infinite loop until hardware reset
  repeat

PUB output_timer(x , y) | temp, tsd
  'deals with all the timer issues, uses the system clock to get a good count

  'time step duration, precalculate to save overhead
  tsd := (clkfreq / time_step)

  repeat while (cnt > (last_time + tsd))
    last_time += tsd
    game_time += 1

  if(cnt < (last_time - tsd)) 'overflow...
    temp := last_time
    repeat until last_time < temp
      last_time += tsd
      game_time += 1

  text.out($A)
  text.out(x)
  text.out($B)
  text.out(y)
  text.out(9) 'clear area
  text.out($A)
  text.out(x)
  text.dec(game_time/time_step)
  text.out(".")
  text.dec(game_time//time_step)

PUB input | temp

  repeat

    lspeed := ADC.average(lvert, 8)
    rspeed := ADC.average(rvert, 8)
    temp := ADC.in(rsel)

    if(temp > 2048)
      button := 0
    else
      button := 1

    temp := ADC.in(lsel)
    if(temp > 2048)
      button &= %01
    else
      button |= %10

    com.tx("y")
    com.tx(button)
    com.tx("z")
    com.tx((lspeed & %1111_1111_0000_0000)>>8)
    com.tx(lspeed & %1111_1111)

    com.tx((rspeed & %1111_1111_0000_0000)>>8)
    com.tx(rspeed & %1111_1111)

    waitcnt(clkfreq/25 + cnt)



PUB tag_known(address) | i
'compares the tag to the list in tags
'also uses known_tags counter

  repeat i from 0 to known_tags
    if strcomp(address, @tags + (i*11))
      return true

  return false

PUB tag_known_add(address) | i
'compares the tag to the list in tags
'also uses known_tags counter
'returns true if the tag is added, false otherwise

  repeat i from 0 to known_tags
    if strcomp(address, @tags + (i*11))
      return false

  bytemove(@tags + ((i-1)*11), address, 11) 'i is 11
  known_tags ++
  return true

PUB output_tags(x, y) | i
  if(known_tags == 0)
    return

  repeat i from 0 to known_tags-1
    {debug.dec(i)
    debug.str(string(": "))
    debug.str(@tags + (i*11))
    debug.tx(13)}

    text.out($A)
    text.out(x)
    text.out($B)
    text.out(y + i)
    text.dec(i)
    text.str(string(": "))
    text.str(@tags + (i*11))

